import { AttributeCellValue } from "./AttributeCellValue";
import { LabelCellValue } from "./LabelCellValue";
import { View } from './View';
/**
 * A abstract definition for small visual area populated with labels and values. Labels are typically presented as simple text phrases.
 * Values are usually presented inside a UI component, such as a TextField or ComboBox.
 */
export class Details extends View {
    findCellValue(matcher) {
        return this.findCellValueForRow((row) => {
            let cellValue = null;
            row.some((cell) => {
                return cell.values.some((value) => {
                    cellValue = matcher(value);
                    return !!cellValue;
                });
            });
            return cellValue;
        });
    }
    /**
     * @param matcher - a function that given a row of cells, returns a matching CellValue (or null)
     */
    findCellValueForRow(matcher) {
        let cellValue = null;
        this.rows.some(row => {
            cellValue = matcher(row);
            return !!cellValue;
        });
        return cellValue;
    }
    getAttributeCellValue(propertyName) {
        if (!this.attributeCellMap) {
            this.initIndexes();
        }
        return this.attributeCellMap[propertyName];
    }
    /**
     * 1-based index
     * @param number
     */
    getConstantByIndex(index) {
        if (!this.constantsArray) {
            this.initIndexes();
        }
        return this.constantsArray.length >= index ? this.constantsArray[index - 1] : null;
    }
    getLabelForProperty(propertyName) {
        if (!this.propertyLabelMap) {
            this.initIndexes();
        }
        return this.propertyLabelMap[propertyName];
    }
    get constants() {
        return this.constantsArray;
    }
    get attributeCells() {
        return Object.keys(this.attributeCellMap).map(key => this.attributeCellMap[key]);
    }
    get labelsByPropName() {
        return this.propertyLabelMap;
    }
    initIndexes() {
        this.attributeCellMap = {};
        this.propertyLabelMap = {};
        this.constantsArray = [];
        this.rows.forEach((row) => {
            const [firstCell, secondCell] = row;
            if (firstCell && firstCell.values) {
                const [labelCellValue] = firstCell.values;
                if (labelCellValue instanceof LabelCellValue) {
                    if (secondCell && secondCell.values) {
                        const [attributeCellValue] = secondCell.values;
                        if (attributeCellValue instanceof AttributeCellValue) {
                            this.propertyLabelMap[attributeCellValue.propertyName] = labelCellValue;
                        }
                    }
                    else {
                        this.constantsArray.push(labelCellValue.value);
                    }
                }
            }
            row.forEach((cell) => {
                cell.values.forEach((cellValue) => {
                    if (cellValue instanceof AttributeCellValue) {
                        this.attributeCellMap[cellValue.propertyName] = cellValue;
                    }
                });
            });
        });
    }
}
